@@ -1328,6 +1328,19 @@ static PyObject *WraptObjectProxy_complex(
 
 /* ------------------------------------------------------------------------- */
 
+static PyObject *WraptObjectProxy_mro_entries(
+        WraptObjectProxyObject *self, PyObject *args, PyObject *kwds)
+{
+    if (!self->wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    return Py_BuildValue("(O)", self->wrapped);
+}
+
+/* ------------------------------------------------------------------------- */
+
 static PyObject *WraptObjectProxy_get_name(
         WraptObjectProxyObject *self)
 {
@@ -1746,6 +1759,10 @@ static PyMethodDef WraptObjectProxy_methods[] = {
     { "__round__",  (PyCFunction)WraptObjectProxy_round, METH_NOARGS, 0 },
 #endif
     { "__complex__",  (PyCFunction)WraptObjectProxy_complex, METH_NOARGS, 0 },
+#if PY_MAJOR_VERSION > 3 || (PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION >= 7)
+    { "__mro_entries__", (PyCFunction)WraptObjectProxy_mro_entries,
+                    METH_VARARGS | METH_KEYWORDS, 0 },
+#endif
     { NULL, NULL },
 };
 
@@ -1944,13 +1961,13 @@ static int WraptPartialCallableObjectProxy_init(
 
     if (!PyObject_Length(args)) {
         PyErr_SetString(PyExc_TypeError,
-		"__init__ of partial needs an argument");
+                "__init__ of partial needs an argument");
         return -1;
     }
 
     if (PyObject_Length(args) < 1) {
         PyErr_SetString(PyExc_TypeError,
-		"partial type takes at least one argument");
+                "partial type takes at least one argument");
         return -1;
     }
 
@@ -1958,7 +1975,7 @@ static int WraptPartialCallableObjectProxy_init(
 
     if (!PyCallable_Check(wrapped)) {
         PyErr_SetString(PyExc_TypeError,
-		"the first argument must be callable");
+                "the first argument must be callable");
         return -1;
     }
 
@@ -1968,7 +1985,7 @@ static int WraptPartialCallableObjectProxy_init(
         return -1;
 
     result = WraptPartialCallableObjectProxy_raw_init(self, wrapped,
-	    fnargs, kwds);
+            fnargs, kwds);
 
     Py_DECREF(fnargs);
 
@@ -2282,12 +2299,15 @@ static PyObject *WraptFunctionWrapperBase_call(
     PyObject *result = NULL;
 
     static PyObject *function_str = NULL;
+    static PyObject *classmethod_str = NULL;
 
     if (!function_str) {
 #if PY_MAJOR_VERSION >= 3
         function_str = PyUnicode_InternFromString("function");
+        classmethod_str = PyUnicode_InternFromString("classmethod");
 #else
         function_str = PyString_InternFromString("function");
+        classmethod_str = PyString_InternFromString("classmethod");
 #endif
     }
 
@@ -2317,8 +2337,10 @@ static PyObject *WraptFunctionWrapperBase_call(
         kwds = param_kwds;
     }
 
-    if (self->instance == Py_None && (self->binding == function_str ||
+    if ((self->instance == Py_None) && (self->binding == function_str ||
             PyObject_RichCompareBool(self->binding, function_str,
+            Py_EQ) == 1 || self->binding == classmethod_str ||
+            PyObject_RichCompareBool(self->binding, classmethod_str,
             Py_EQ) == 1)) {
 
         PyObject *instance = NULL;
@@ -2493,6 +2515,74 @@ static PyObject *WraptFunctionWrapperBase_descr_get(
 
 /* ------------------------------------------------------------------------- */
 
+static PyObject *WraptFunctionWrapperBase_set_name(
+        WraptFunctionWrapperObject *self, PyObject *args, PyObject *kwds)
+{
+    PyObject *method = NULL;
+    PyObject *result = NULL;
+
+    if (!self->object_proxy.wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    method = PyObject_GetAttrString(self->object_proxy.wrapped,
+            "__set_name__");
+
+    if (!method) {
+        PyErr_Clear();
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+
+    result = PyObject_Call(method, args, kwds);
+
+    Py_DECREF(method);
+
+    return result;
+}
+
+/* ------------------------------------------------------------------------- */
+
+static PyObject *WraptFunctionWrapperBase_subclasscheck(
+        WraptFunctionWrapperObject *self, PyObject *args)
+{
+    PyObject *subclass = NULL;
+    PyObject *object = NULL;
+    PyObject *result = NULL;
+
+    int check = 0;
+
+    if (!self->object_proxy.wrapped) {
+      PyErr_SetString(PyExc_ValueError, "wrapper has not been initialized");
+      return NULL;
+    }
+
+    if (!PyArg_ParseTuple(args, "O", &subclass))
+        return NULL;
+
+    object = PyObject_GetAttrString(subclass, "__wrapped__");
+
+    if (!object)
+        PyErr_Clear();
+
+    check = PyObject_IsSubclass(object ? object: subclass,
+            self->object_proxy.wrapped);
+
+    Py_XDECREF(object);
+
+    if (check == -1)
+        return NULL;
+
+    result = check ? Py_True : Py_False;
+
+    Py_INCREF(result);
+
+    return result;
+}
+
+/* ------------------------------------------------------------------------- */
+
 static PyObject *WraptFunctionWrapperBase_get_self_instance(
         WraptFunctionWrapperObject *self, void *closure)
 {
@@ -2563,6 +2653,16 @@ static PyObject *WraptFunctionWrapperBase_get_self_parent(
 
 /* ------------------------------------------------------------------------- */;
 
+static PyMethodDef WraptFunctionWrapperBase_methods[] = {
+    { "__set_name__", (PyCFunction)WraptFunctionWrapperBase_set_name,
+                    METH_VARARGS | METH_KEYWORDS, 0 },
+    { "__subclasscheck__", (PyCFunction)WraptFunctionWrapperBase_subclasscheck,
+                    METH_VARARGS, 0 },
+    { NULL, NULL },
+};
+
+/* ------------------------------------------------------------------------- */;
+
 static PyGetSetDef WraptFunctionWrapperBase_getset[] = {
     { "__module__",         (getter)WraptObjectProxy_get_module,
                             (setter)WraptObjectProxy_set_module, 0 },
@@ -2616,7 +2716,7 @@ PyTypeObject WraptFunctionWrapperBase_Type = {
     offsetof(WraptObjectProxyObject, weakreflist), /*tp_weaklistoffset*/
     0,                      /*tp_iter*/
     0,                      /*tp_iternext*/
-    0,                      /*tp_methods*/
+    WraptFunctionWrapperBase_methods, /*tp_methods*/
     0,                      /*tp_members*/
     WraptFunctionWrapperBase_getset, /*tp_getset*/
     0,                      /*tp_base*/
